#pragma rtGlobals=1		// Use modern global access method.

//	Touchstone file loader
//  	********************************************************************
//	* Loads *.s4p files automatically, adds button to the Load Wave Menu
//  	* The user selects if the ports are single-ended, differential, or not used
//  	* The S-parameters are then imported as either SE -> SE, SE -> BAL, SE-SE -> BAL, or BAL -> BAL
//	* The Touchstone file format was originated by Agilent Corporation to store n-port parameter data.
//	* This file loader is based on the following draft specification:
//	*	http://www.eda.org/pub/ibis/connector/touchstone_spec11.pdf
//  	* Code base modified from original Touchstone File Loader created by Howard Rodstein. 
//  
//   ********************************************************************
Constant kTouchStoneFileLoaderVersion = 1.00

// Add an item to Igor's Load Waves submenu (Data menu)
Menu "Load Waves"
	"Load Touchstone File...", /Q, LoadTouchstoneFileMenuItem() //Adds "Load Touchstone File" button to menu
End

Menu "Macros"
	"View Touchstone Loader Help", /Q, DisplayHelpTopic "Touchstone File Loader" //Adds button to help docs
End

// These are used when displaying data from one file only.
static Constant kGraphModeUndefined = -1
static Constant kGraphModeAllInOne = 1					// Display all values in a single graph
static Constant kGraphModeValuesSeparate = 2			// Display each value in a separate graph
static Constant kGraphModePairsSeparate = 3			// Display each value pair in a separate graph

Structure TouchstoneFileInfo
	Variable numberOfPorts				// 1, 2, 3, 4 . . . This is deduced from the file name.

	// These come from the option line which (for example) looks something like this:
	//		# GHZ S DB R 50
	String frequencyUnit				// "GHZ", "MHZ", "KHZ", etc.
	String parameterKind				// "S" (S-parameter), "Y" (admittance),"Z" (Z-parameter), "H" (hybrid), "G" (hybrid-G)
	String format						// "DB" (dB-angle), "MA" (magnitude-angle), "RI" (real-imaginary)
	Variable reference					// Reference resistance in ohms
EndStructure

static Function StoreTouchstoneFileInfo(fileInfo)		// Stores info as variables in current data folder.
	STRUCT TouchstoneFileInfo &fileInfo

	Variable/G numberOfPorts = fileInfo.numberOfPorts
	String/G frequencyUnit = fileInfo.frequencyUnit
	String/G parameterKind = fileInfo.parameterKind
	String/G format = fileInfo.format
	Variable/G reference = fileInfo.reference
End

static Function RetrieveTouchstoneFileInfo(fileInfo)		// Retrives info from variables in current data folder.
	STRUCT TouchstoneFileInfo &fileInfo
	
	NVAR numberOfPorts, reference
	SVAR frequencyUnit, parameterKind, format

	fileInfo.numberOfPorts = numberOfPorts
	fileInfo.frequencyUnit = frequencyUnit
	fileInfo.parameterKind = parameterKind
	fileInfo.format = format
	fileInfo.reference = reference
End

static Function/S GetNextSymbol(text, pos)
	String text
	Variable &pos						// Input and output
	
	String symbol = ""
	
	Variable textLen = strlen(text)
	
	Variable ch

	do
		ch = char2num(text[pos])
		if (ch > 0x20)					// Past all white space and control characters?
			break
		endif
		pos += 1
	while(pos < textLen)

	// Now deposit characters into symbol.
	if (pos < textLen)
		do
			ch = char2num(text[pos])
			
			if (ch <= 0x20)												// End of symbol?
				break
			endif
			
			symbol += text[pos]
			pos += 1
		while(pos < textLen)
	endif

	return symbol
End

static Function/S ParseOptionLine(optionStr, fileInfo, quiet)
	String optionStr
	STRUCT TouchstoneFileInfo &fileInfo
	Variable quiet

	fileInfo.frequencyUnit = "GHZ"
	fileInfo.parameterKind = "S"
	fileInfo.format = "MA"
	fileInfo.reference = 50

	// Symbols in the option line can appear in any order which greatly complicates interpreting it
	Variable pos = 1					// 1 to skip the initial # symbol
	do
		String symbol = GetNextSymbol(optionStr, pos)
		if (strlen(symbol) == 0)
			break
		endif
		
		strswitch(symbol)
			case "GHZ":
				fileInfo.frequencyUnit = symbol
					break
			
			case "MHZ":
				fileInfo.frequencyUnit = symbol
				break

			case "KHZ":
				fileInfo.frequencyUnit = symbol
				break

			case "HZ":
				fileInfo.frequencyUnit = symbol
				break
			
			case "S":
			case "Y":
			case "Z":
			case "H":
			case "G":
				fileInfo.parameterKind = symbol
				break
				
			case "DB":
			case "MA":
			case "RI":
				fileInfo.format = symbol
				break
				
			case "R":
				symbol = GetNextSymbol(optionStr, pos)
				fileInfo.reference = str2num(symbol)
				break
			
			default:
				if (quiet != 2)
					Printf "Unknown symbol in option line: \"%s\"\r", symbol
				endif
				break
		endswitch	
	while(1)
End

static Function ExtractTouchstoneOptionInfo(refNum, fileInfo, quiet)
	Variable refNum			// File reference number
	STRUCT TouchstoneFileInfo &fileInfo
	Variable quiet				// 0 = print diagnostic and error messages, 1 = print error messages only, 2 = do not print diagnostic or error messages
	
	Variable result = 0
	
	// Set defaults used if option line is missing.
	fileInfo.frequencyUnit = "GHZ"
	fileInfo.parameterKind= "S"
	fileInfo.format = "MA"
	fileInfo.reference = 50
	
	// Read option string
	String optionStr = ""
	FSetPos refNum, 0
	Variable done = 0
	do
		FReadLine refNum, optionStr
		if (strlen(optionStr) == 0)
			Close refNum
			return -1								// Unexpected end of file
		endif
		
		strswitch(optionStr[0])
			case "#":								// Option line?
				ParseOptionLine(optionStr, fileInfo, quiet)
				done = 1
				break
			
			case "!":								// Comment line?
				break
			
			default:									// Should be data line.
				done = 1
				break
		endswitch
	while(!done)
	
	return result
End

static Function SkipOptionAndCommentLines(refNum, dataStartLine, dataStartPos, quiet)
	Variable refNum			// File reference number
	Variable &dataStartLine
	Variable &dataStartPos		// Start of data as file position relative to start of file
	Variable quiet				// 0 = print diagnostic and error messages, 1 = print error messages only, 2 = do not print diagnostic or error messages
	
	dataStartLine = 0
	dataStartPos = 0
	
	Variable result = 0
	
	String line
	Variable done = 0
	do
		FStatus refNum
		dataStartPos = V_filePos
		FReadLine refNum, line
		if (strlen(line) == 0)
			Close refNum
			return -1								// Unexpected end of file
		endif
		
		strswitch(line[0])
			case "#":								// Option line?
				dataStartLine +=  1
				break
			
			case "!":								// Comment line?
				dataStartLine +=  1
				break
			
			default:									// Should be data line.
				FSetPos refNum, dataStartPos
				done = 1
				break
		endswitch
	while(!done)
	
	return result
End

static Function SetTouchstoneViewPrefs(createTable, createGraph, graphMode)
	Variable createTable, createGraph, graphMode
	
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:TouchstoneLoader
	
	Variable/G root:Packages:TouchstoneLoader:createTable = createTable
	Variable/G root:Packages:TouchstoneLoader:createGraph = createGraph
	Variable/G root:Packages:TouchstoneLoader:graphMode = graphMode
End

static Function GetTouchstoneViewPrefs(createTable, createGraph, graphMode)
	Variable &createTable, &createGraph, &graphMode						// Outputs
	
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:TouchstoneLoader
	
	createTable = 1
	NVAR/Z gCreateTable = root:Packages:TouchstoneLoader:createTable
	if (NVAR_Exists(gCreateTable))
		createTable =  gCreateTable
	endif
	if ( (createTable<0) || (createTable>3) )
		createTable = 1
	endif
	
	createGraph = 1
	NVAR/Z gCreateGraph = root:Packages:TouchstoneLoader:createGraph
	if (NVAR_Exists(gCreateGraph))
		createGraph =  gCreateGraph
	endif
	if ( (createGraph<0) || (createGraph>3) )
		createGraph = 1
	endif
	
	graphMode = kGraphModeAllInOne
	NVAR/Z gGraphMode = root:Packages:TouchstoneLoader:graphMode
	if (NVAR_Exists(gGraphMode))
		graphMode =  gGraphMode
	endif
	if ( (graphMode<kGraphModeAllInOne) || (graphMode>kGraphModePairsSeparate) )
		graphMode = kGraphModeAllInOne
	endif
End

//	ViewTouchstoneFileDialog(title, createTable, createGraph, graphMode)
//	Displays a dialog in which the user can enter parameters.
//	This is called when we are displaying Touchstone data from a single file.
//	createTable, createGraph and graphMode are outputs only.
//	The function result is 0 if OK or -1 if cancel.
Function ViewTouchstoneFileDialog(title, createTable, createGraph, graphMode)
	String title					// Dialog title
	Variable &createTable		// Output. 0 = no table, 1 = edit even value column, 2 = edit odd value columns, 3 = edit both
	Variable &createGraph		// Output. 0 = no graph, 1 = graph even value column, 2 = graph odd value columns, 3 = graph both
	Variable &graphMode		// Output. kGraphModeAllInOne, kGraphModeValuesSeparate or kGraphModePairsSeparate

	GetTouchstoneViewPrefs(createTable, createGraph, graphMode)
	
	Variable createTable2 = createTable + 1
	Variable createGraph2 = createGraph + 1
	Variable graphMode2 = graphMode
	
	Prompt createTable2, "Create table", popup "No table;Even value columns;Odd value columns;All value columns;"
	Prompt createGraph2, "Create graph", popup "No graph;Even value columns;Odd value columns;All value columns;"
	Prompt graphMode2, "Graph mode", popup "All values in one graph;Each value in a separate graph;Each pair of values in a separate graph;"
	DoPrompt title, createTable2, createGraph2, graphMode2
	if (V_Flag != 0)
		return -1			// User cancelled.
	endif
	
	createTable = createTable2 - 1
	createGraph = createGraph2 - 1
	graphMode = graphMode2

	SetTouchstoneViewPrefs(createTable, createGraph, graphMode)
	
	return 0
End

static Function/S TouchstoneFreqWaveName(frequencyUnit)
	String frequencyUnit
       SVAR gfileName=gfileName
	String freqName= gfileName + "_freq_" + frequencyUnit		// e.g., freq_GHZ
	
	return freqName
End

static Function/S TouchstoneValueWaveName(numberOfPorts, parameterKind, format, valueIndex)
	Variable numberOfPorts
	String parameterKind
	String format
	Variable valueIndex				// Zero-based index into the columns of parameter values.
	
	String suffix 
	String name
	
	if (valueIndex >= 2*numberOfPorts^2)
		return "Bad valueIndex parameter"					// Should never happen
	endif

	Variable index
	Variable first
	Variable second

	index = trunc(valueIndex / 2)
	if (numberOfPorts <= 2)
		first = mod(index,numberOfPorts) + 1
		second = trunc(index/numberOfPorts)+1
	else
		first = trunc(index/numberOfPorts)+1
		second = mod(index,numberOfPorts) + 1
	endif

	suffix = num2istr(first) + num2istr(second)
	SVAR gfileName=gfileName

	name = gfileName + "_" +parameterKind + suffix + "ss"
	
	
	strswitch(format)
		case "DB":
			if (valueIndex & 1)
				name += "_A"
			else
				name += "_DB"
			endif
			break
			
		case "MA":
			if (valueIndex & 1)
				name += "_A"
			else
				name += "_M"
			endif
			break
			
		case "RI":
			if (valueIndex & 1)
				name += "_I"
			else
				name += "_R"
			endif
			break
			
		case "":
			break					// Caller does not want format appended.
	endswitch
	
	return name
End

static Function SkipWhiteSpace(buffer, bufferLen, pos)
	String buffer
	Variable bufferLen
	Variable &pos						// Input and output
	
	Variable ch

	if (pos < bufferLen)
		do
			ch = char2num(buffer[pos])
			
			if (ch == 0x21)											// Start of a comment?
				pos = bufferLen											// Skip the rest of the line.
				break
			endif
			
			if (ch==0x2B || ch==0x2D || ch >= 0x30)					// A plus, minus or digit will pass this test
				break
			endif
			pos += 1					// Skip space, tab, CR or LF
		while(pos < bufferLen)
	endif
End

static Function ReadValue(buffer, bufferLen, pos, val)
	String buffer
	Variable bufferLen
	Variable &pos						// Input and output
	Variable &val						// Output
	
	Variable err = 0
	
	val = str2num(buffer[pos, bufferLen])
	
	Variable ch

	// Now skip this number
	do
		ch = char2num(buffer[pos])
		if (ch==0x20 || ch==0x09 || ch==0x0D || ch==0x0A)			// A space, tab, CR or LF will pass this test
			break
		endif
		pos += 1
	while(pos < bufferLen)
	
	SkipWhiteSpace(buffer, bufferLen, pos)
	
	return err
End

static Function FindXScaling(freq, x0, dx)
	Wave freq
	Variable &x0, &dx				// Outputs
	
	x0 = freq[0]
	dx = freq[1] - freq[0]
	
	Variable epsilon = abs(x0 / 10000)
	
	// Now make sure that frequencies are really evenly spaced.
	Variable numPoints = numpnts(freq)
	Variable i
	for(i=1; i<numPoints-2; i+=1)
		Variable currentDX = freq[i+1]-freq[i]
		if (abs(currentDX-dx) > epsilon)
			x0 = 0						// Signal that frequencies are not evenly-spaced
			dx = 0
		endif	
	endfor
End

static Function LoadTouchstoneData(fileInfo, refNum, dataStartPos, dataStartLine, quiet)
	STRUCT TouchstoneFileInfo &fileInfo
	Variable refNum
	Variable dataStartPos			// Start of data as file position relative to start of file
	Variable dataStartLine			// Zero-based
	Variable quiet					// 0 = print diagnostic and error messages, 1 = print error messages only, 2 = do not print diagnostic or error messages

	String name
	
	Variable numberOfFrequencies = 0
	Variable numberOfPointsPerWave =  1024				// Initial size of waves. They grow as needed.

	Variable numberOfValuesPerFreq
	numberOfValuesPerFreq = (fileInfo.numberOfPorts ^ 2) * 2
	
	// Make the frequency wave
	String freqName= TouchstoneFreqWaveName(fileInfo.frequencyUnit)		// e.g., freq_GHZ
	Make/O/N=(numberOfPointsPerWave) $freqName
	
	// Make the value waves
	Variable valueColumnIndex
	for(valueColumnIndex=0; valueColumnIndex<numberOfValuesPerFreq; valueColumnIndex+=1)
		name = TouchstoneValueWaveName(fileInfo.numberOfPorts, fileInfo.parameterKind, fileInfo.format, valueColumnIndex)
		Make/O/N=(numberOfPointsPerWave) $name
	endfor

	Variable lineNumber = dataStartLine - 1
	Variable pointNum						// Point we are currently working on.
	Variable waveIndex						// -1 for frequency wave, 0 for s11, 1 for s12, etc.
	String buffer
	Variable bufferLen
	Variable pos							// Next position in buffer to read from.
	Variable val
	Variable err = 0
	
	buffer = ""
	bufferLen = 0
	pos = 0
	waveIndex = -1
	pointNum = 0
	do
		if (pos >= bufferLen)
			do												// Read till we get a non-blank line.
				lineNumber += 1
				
				FReadLine refNum, buffer
				bufferLen = strlen(buffer)
				if (bufferLen == 0)
					break									// Hit end of file.
				endif
			
				if (CmpStr(buffer[0], "!") == 0)			// Is this a comment line?
					if (strsearch(buffer, "Noise Parameters", 0) > 0)	// Start of optional noise parameters?
						bufferLen = 0									// Signal end of file.
						break											// We do not support loading of noise parameters at this time.
					endif
					continue								// Skip comment line.
				endif
			
				pos = 0
				SkipWhiteSpace(buffer, bufferLen, pos)		// Skip any leading white space.
				if (pos < bufferLen)							// Got some non-white space characters?
					break
				endif
			while(1)
		endif

		if (bufferLen == 0)
			break											// Hit end of file.
		endif
		
		err = ReadValue(buffer, bufferLen, pos, val)
		if (err != 0)
			break
		endif
		
		// Check if we need to increase size of waves
		if (pointNum >= numberOfPointsPerWave)
			numberOfPointsPerWave += 1024
			Wave w = $freqName
			Redimension/N=(numberOfPointsPerWave) w
			for(valueColumnIndex=0; valueColumnIndex<numberOfValuesPerFreq; valueColumnIndex+=1)
				name = TouchstoneValueWaveName(fileInfo.numberOfPorts, fileInfo.parameterKind, fileInfo.format, valueColumnIndex)
				Wave w = $name
				Redimension/N=(numberOfPointsPerWave) w
			endfor
		endif
		
		if (waveIndex < 0)
			Wave w = $freqName
			w[pointNum] = val
			waveIndex = 0
		else
			name = TouchstoneValueWaveName(fileInfo.numberOfPorts, fileInfo.parameterKind, fileInfo.format, waveIndex)
			Wave w = $name
			w[pointNum] = val
			waveIndex += 1
			if (waveIndex >= numberOfValuesPerFreq)
				waveIndex = -1
				pointNum += 1
			endif
		endif
	while(1)
	
	Variable numPointsLoaded = pointNum + 1
	if (waveIndex < 0)
		// pointNum is number of  NEXT point to be loaded
		numPointsLoaded -= 1
	endif

	// Delete any unsed points in waves
	if (numberOfPointsPerWave > numPointsLoaded)
		Wave w = $freqName
		Redimension/N=(numPointsLoaded) w
		for(valueColumnIndex=0; valueColumnIndex<numberOfValuesPerFreq; valueColumnIndex+=1)
			name = TouchstoneValueWaveName(fileInfo.numberOfPorts, fileInfo.parameterKind, fileInfo.format, valueColumnIndex)
			Wave w = $name
			Redimension/N=(numPointsLoaded) w
		endfor
	endif
	
	// Set wave scaling and units

	SetScale /P d 0, 0, ""+fileInfo.frequencyUnit, $freqName	// Set data units of frequency wave
	
	Variable x0, dx												// Set X units of value waves if frequencies are evenly spaced.
	FindXScaling($freqName, x0, dx)
	if (dx != 0)													// Freq is evenly spaced?
		for(valueColumnIndex=0; valueColumnIndex<numberOfValuesPerFreq; valueColumnIndex+=1)
			name = TouchstoneValueWaveName(fileInfo.numberOfPorts, fileInfo.parameterKind, fileInfo.format, valueColumnIndex)
			Wave w = $name
			SetScale /P x x0, dx, ""+fileInfo.frequencyUnit, w
		endfor
	endif
	
	for(valueColumnIndex=0; valueColumnIndex<numberOfValuesPerFreq; valueColumnIndex+=1)	// Set data units for value waves
		name = TouchstoneValueWaveName(fileInfo.numberOfPorts, fileInfo.parameterKind, fileInfo.format, valueColumnIndex)
		Wave w = $name
		String units
		strswitch(fileInfo.format)
			case "DB":
				if (valueColumnIndex & 1)
					units = "deg"
				else
					units = "DB"
				endif
				break
				
			case "MA":
				if (valueColumnIndex & 1)
					units = "deg"
				else
					units = ""									// I'm not sure what units should be here.
				endif
				break
				
			case "RI":
				if (valueColumnIndex & 1)
					units = ""									// I'm not sure what units should be here.
				else
					units = ""									// I'm not sure what units should be here.
				endif
				break
		endswitch
		SetScale /P d 0, 0, units, w
	endfor

	return 0
End

Function LoadTouchstoneFile(pathName, fileName, createTable, createGraph, graphMode, quiet)
	String pathName			// Igor symbolic path name or "" for dialog.
	String fileName				// file name, partial path and file name, or full path or "" for dialog.
	Variable createTable		// 0 = no table, 1 = edit even value column, 2 = edit odd value columns, 3 = edit both, 4 for dialog.
	Variable createGraph		// 0 = no graph, 1 = graph even value column, 2 = graph odd value columns, 3 = graph both, 4 for dialog.
	Variable graphMode			// kGraphModeAllInOne, kGraphModeValuesSeparate or kGraphModePairsSeparate, kGraphModeUndefined for dialog
	Variable quiet				// 0 = print diagnostic and error messages, 1 = print error messages only, 2 = do not print diagnostic or error messages
	
	Variable err
	
	if ( (createTable==4) || (createGraph==4) || (graphMode==kGraphModeUndefined))
		err = ViewTouchstoneFileDialog("Load Touchstone File", createTable, createGraph, graphMode)
		if (err != 0)
			return err
		endif
	endif
	
	
	
	
	// This puts up a dialog if the pathName and fileName do not specify a file.
	String message = "Select a Touchstone file"
	Variable refNum
	Open /R /Z=2 /P=$pathName /T="????" /M=message refNum as fileName	// /T flag presents "All files" in Open File dialog.
	
	// Save outputs from Open in a safe place.
	err = V_Flag
	String fullPath = S_fileName
	fileName = ParseFilePath(0, fullPath, ":", 1, 0)			// In case user chose a different file 	through dialog displayed by Open.
	String/G gfileName=CleanupName(fileName, 0)
	//shorten filename or else max string length will be met
	if (strlen(gfileName) > 15)
	variable j=0
	string temp=""
	do
	temp[j]=gfileName[j]
	j+=1
	while(j<15)
	gfileName=temp
	endif

		
	if (err != 0)
		return err			// -1 means user canceled.
	endif

	STRUCT TouchstoneFileInfo fileInfo

	// Determine the number of ports from the file name extension
	String extension
	extension = ParseFilePath(4, fileName, ":", 0, 0)		// Should be s<n>p where <n> is the number of ports
	Variable len = strlen(extension)
	if (len < 3)
		err = -1
	else
		if (CmpStr(extension[0], "s") != 0)
			err = -1
		else
			if (CmpStr(extension[len-1], "p") != 0)
				err = -1
			else
				Variable numberOfPorts = str2num(extension[1,len-2])
				if (numberOfPorts<1 || numberOfPorts>100)
					err = -1
				else
					fileInfo.numberOfPorts = numberOfPorts
				endif
			endif
		endif
	endif
	if (err != 0)
		if (quiet != 2)
			Print "Bad file name extension - expected \".s<n>p\" where <n> is number of ports."
		endif
		Close refNum
		return -1
	endif

	// Load the option line information
	if (ExtractTouchstoneOptionInfo(refNum, fileInfo, quiet!=2) != 0)
		if (quiet != 2)
			Print "Error while loading option line of the file."
		endif
		Close refNum
		return -1
	endif
	
	Variable dataStartLine
	Variable dataStartPos			// Start of data as file position relative to start of file
	if (SkipOptionAndCommentLines(refNum, dataStartLine, dataStartPos, quiet!=2) != 0)
		if (quiet != 2)
			Print "Error while skipping option and comment lines in the file."
		endif
		Close refNum
		return -1
	endif
	
	// Create data folder for waves
//	String dfName = ParseFilePath(3, fileName, ":", 0, 0)
//	dfName = CleanupName(dfName, 0)
//	NewDataFolder /O /S $dfName
	String fullDFPath = GetDataFolder(1)
	
	if (quiet == 0)
		Printf "Loading Touchstone data from \"%s\"\r", fullPath
	endif
	
	err = LoadTouchstoneData(fileInfo, refNum, dataStartPos, dataStartLine, quiet)

	Close refNum
	
	if (err == 0)
		StoreTouchstoneFileInfo(fileInfo)
	endif
	
	//SetDataFolder ::					// Restore original current data folder
	
	if (err != 0)
		return err
	endif
	
//	if (quiet == 0)
//		Variable numWavesCreated = 2*fileInfo.numberOfPorts^2 + 1
//		Printf "Created %d waves in data folder %s\r", numWavesCreated, GetDataFolder(1)+dfName
//	endif

	

	return 0			// Zero signifies no error.	
End

Function LoadTouchstoneFileMenuItem()
	//LoadTouchstoneFile("", "", 4, 4, 4, 0)
	LoadTouchstoneFile("", "", 0, 0, 0, 0) // **Change : Removes dialog the popped up prior to selecting Touchstone file
End

static Function IsValidTouchstoneExtension(extension)
	String extension

	Variable isValidExtension = 0
	if (strlen(extension) == 3)							// TO DO: Change to allow extensions greater than three characters (e.g., s10p, s11p, etc.)
		if (CmpStr(extension[0], "s") == 0)
			if (CmpStr(extension[2], "p") == 0)
				isValidExtension = 1
			endif
		endif
	endif

	return isValidExtension
End

//	LoadAllTouchstoneFilesDialog(extension, createTable, createGraph)
//	Displays a dialog in which the user can enter parameters.
//	All of the parameters function both as inputs, to preset the dialog values,
//	and as output, to return the values to the calling routine.
//	The function result is 0 if OK or -1 if cancel.
Function LoadAllTouchstoneFilesDialog(extension, createTable, createGraph)
	String &extension					// Input and output
	Variable &createTable				// Input and output.
	Variable &createGraph				// Input and output.

	Variable isValidExtension = IsValidTouchstoneExtension(extension)

	if (!isValidExtension && CmpStr(extension,"????")!=0)
		extension = ".s2p"
	endif
	if ( (createTable<1) || (createTable>3) )
		createTable = 1
	endif
	if ( (createGraph<1) || (createGraph>3) )
		createGraph = 1
	endif
	
	// This is necessary because you can't pass an & parameter to Prompt.
	String extension2 = extension
	Variable createTable2 = createTable
	Variable createGraph2 = createGraph
	
	Prompt extension2, "Touchstone file name extension", popup ".s1p;.s2p;.s3p;.s4p;.s5p;.s6p;.s7p;.s8p;.s9p;Any extension"
	Prompt createTable2, "Create table", popup "No table;Even value columns;Odd value columns;All value columns;"
	Prompt createGraph2, "Create graph", popup "No graph;Even value columns;Odd value columns;All value columns;"
	DoPrompt "Load All Touchstone Files", extension2, createTable2, createGraph2
	if (V_Flag != 0)
		return -1			// User cancelled.
	endif
	
	if (CmpStr(extension2, "Any extension") == 0)
		extension2 = "????"
	endif
	
	extension = extension2
	createTable = createTable2 - 1
	createGraph = createGraph2 - 1
	
	return 0
End














